module RestResource
  def self.append_features(base)
    super
    base.extend(ClassMethods)
  end
  module ClassMethods
    def authored_by(name, options = {})
      write_inheritable_hash(:author, {:name => name, :email => options[:email], :uri => options[:uri]})
    end
    def rest_resource(model_id, options = {})
      model_name = model_id.to_s
      write_inheritable_array(:models, [model_name])
      class_name  = options[:class_name] || model_name.camelize
      template_path_from_view = "../../vendor/plugins/rest_resource/templates"
      
      module_eval <<-"end_eval"
        def #{model_name}
          #logger.info request.env.inspect
          unless #{options[:updated].inspect}.blank?
            @updated_name = "#{options[:updated]}"
          else
            if #{class_name}.column_names.include?('updated_at')
              @updated_name = 'updated_at'
            elsif #{class_name}.column_names.include?('updated_on')
              @updated_name = 'updated_on'
            end
          end
          unless #{options[:published].inspect}.blank?
            @published_name = "#{options[:published]}"
          else
            if #{class_name}.column_names.include?('created_at')
              @published_name = 'created_at'
            elsif #{class_name}.column_names.include?('created_on')
              @published_name = 'created_on'
            end
          end
          #logger.info '@updated_name: ' + @updated_name.inspect
          order_sql = @updated_name ? @updated_name + ' DESC' : nil
          
          if params[:id] # Member URI
            if params[:id] == 'opensearchdescription'
              case request.method
              when :get
                # OpenSearch Description Data
                @model_name = "#{model_name}"
                headers['Content-Type'] = 'application/opensearchdescription+xml'
                render :template => "#{template_path_from_view}/opensearchdescription", :layout => false
              else
                headers['Allow'] = 'GET'
                render :nothing => true, :status => 405
              end
            else
          
              if #{class_name}.exists?(params[:id])
                @resource = #{class_name}.find(params[:id])
    
                case request.method
                when :get
                  # Retrieving a Resource
                  respond_to do |type|
                    type.xml {
                      headers['Content-Type'] = 'application/xml'
                      render :text => @resource.to_xml
                    }
                    type.yaml {
                      headers['Content-Type'] = 'application/x-yaml'
                      render :text => @resource.to_yaml
                    }
                    type.atom {
                      headers['Content-Type'] = 'application/atom+xml'
                      render :template => "#{template_path_from_view}/atom", :layout => false
                    }
                    type.js {
                      headers['Content-Type'] = 'text/javascript'
                      render :text => @resource.to_json
                    }
                  end
                when :put
                  # Updating a Resource
                  if @resource.update_attributes(params["#{model_id}".to_sym])
                    render :nothing => true, :status => 200
                  else
                    render :text => @resource.errors.full_messages.join("\n"), :status => 406
                  end
                when :delete
                  # Deleting a Resource
                  #{class_name}.destroy @resource.id
                  render :nothing => true, :status => 204
                end
              else
                render :nothing => true, :status => 404
              end
            end
          
          else # Collection URI
            case request.method
            when :get
              # Listing Collection Members (with Query)
              conditions = ["body LIKE ?", '%' + params[:q] + '%'] if params[:q]
              @pages, @resources = paginate("#{model_id}".to_sym, 
                                              :conditions => conditions,
                                              :order => order_sql,
                                              :per_page => 10)
              @model_name = "#{model_name}"
              @last_updated = #{class_name}.find(:first, :order => order_sql).send(@updated_name) if @updated_name
              @title = #{options[:title].inspect}.blank? ? "#{model_name}".camelize : "#{options[:title]}"
              @author = self.class.read_inheritable_attribute(:author) || {}
              headers['Content-Type'] = 'application/atom+xml'
              render :template => "#{template_path_from_view}/collection", :layout => false
            when :post
              # Creating a Resource
              new_resource = #{class_name}.new(params["#{model_id}".to_sym])
              if new_resource.save
                headers['Status'] = '201 Created'
                redirect_to :id => new_resource.id
              else
                render :text => new_resource.errors.full_messages.join("\n"), :status => 406
              end
            end
          end
        end

        def index
          case request.method
          when :get
            # Introspection Documents
            @models = self.class.read_inheritable_attribute(:models)
            headers['Content-Type'] = 'application/atomserv+xml'
            render :template => "#{template_path_from_view}/introspection", :layout => false
          else
            headers['Allow'] = 'GET'
            render :nothing => true, :status => 405
          end
        end
      end_eval
    end
  end
end

#module Mime
#  JSON = Type.new "application/json", :json
#  LOOKUP["application/json"] = JSON
#end 
